@@ -4,6 +4,8 @@ import qualified Data.Map.Strict as Map
 
 import Sound.Tidal.Pattern
 import Sound.Tidal.Utils
+import Data.Maybe (fromMaybe)
+import Data.Word (Word8)
 
 -- | group multiple params into one
 grp :: [String -> ControlMap] -> Pattern String -> ControlPattern
@@ -14,10 +16,12 @@ grp fs p = splitby <$> p
         split = wordsBy (==':')
 
 mF :: String -> String -> ControlMap
-mF name v = Map.singleton name (VF $ read v)
+mF name v = fromMaybe Map.empty $ do f <- readMaybe v
+                                     return $ Map.singleton name (VF f)
 
 mI :: String -> String -> ControlMap
-mI name v = Map.singleton name (VI $ read v)
+mI name v = fromMaybe Map.empty $ do i <- readMaybe v
+                                     return $ Map.singleton name (VI i)
 
 mS :: String -> String -> ControlMap
 mS name v = Map.singleton name (VS v)
@@ -33,6 +37,9 @@ s = sound
 cc :: Pattern String -> ControlPattern
 cc = grp [mF "ccn", mF "ccv"]
 
+nrpn :: Pattern String -> ControlPattern
+nrpn = grp [mI "nrpn", mI "val"]
+
 -- | Singular params
 
 pF :: String -> Pattern Double -> ControlPattern
@@ -44,6 +51,9 @@ pI name = fmap (Map.singleton name . VI)
 pS :: String -> Pattern String -> ControlPattern
 pS name = fmap (Map.singleton name . VS)
 
+pX :: String -> Pattern [Word8] -> ControlPattern
+pX name = fmap (Map.singleton name . VX)
+
 -- | patterns for internal sound routing
 toArg :: Pattern String -> ControlPattern
 toArg = pS "toArg"
@@ -166,6 +176,9 @@ dry = pF "dry"
 end :: Pattern Double -> ControlPattern
 end = pF "end"
 
+freq :: Pattern Double -> ControlPattern
+freq = pF "freq"
+
 -- | a pattern of numbers that specify volume. Values less than 1 make
 -- the sound quieter. Values greater than 1 make the sound louder. For
 -- the linear equivalent, see @amp@.
@@ -274,6 +287,13 @@ ophatdecay = pF "ophatdecay"
 -}
 orbit :: Pattern Int -> ControlPattern
 orbit = pI "orbit"
+
+overgain :: Pattern Double -> ControlPattern
+overgain = pF "overgain"
+
+overshape :: Pattern Double -> ControlPattern
+overshape = pF "overshape"
+
 -- | a pattern of numbers between 0 and 1, from left to right (assuming stereo), once round a circle (assuming multichannel)
 pan :: Pattern Double -> ControlPattern
 pan = pF "pan"
@@ -540,8 +560,8 @@ drumN _ = 0
 
 -- SuperDirt MIDI Params
 
-array :: Pattern Double -> ControlPattern
-array = pF "array"
+array :: Pattern [Word8] -> ControlPattern
+array = pX "array"
 midichan :: Pattern Double -> ControlPattern
 midichan = pF "midichan"
 control :: Pattern Double -> ControlPattern
@@ -552,6 +572,20 @@ ccn = pF "ccn"
 ccv :: Pattern Double -> ControlPattern
 ccv = pF "ccv"
 
+polyTouch :: Pattern Double -> ControlPattern
+polyTouch = pF "polyTouch"
+
+midibend :: Pattern Double -> ControlPattern
+midibend = pF "midibend"
+
+miditouch :: Pattern Double -> ControlPattern
+miditouch = pF "miditouch"
+
+nrpnn :: Pattern Int -> ControlPattern
+nrpnn = pI "nrpn"
+nrpnv :: Pattern Int -> ControlPattern
+nrpnv = pI "val"
+
 ctlNum :: Pattern Double -> ControlPattern
 ctlNum = pF "ctlNum"
 
