@@ -1,4 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable, TypeSynonymInstances, FlexibleInstances #-}
+{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, TypeSynonymInstances #-}
 {-# LANGUAGE DeriveFunctor #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
@@ -7,19 +7,20 @@ module Sound.Tidal.Pattern where
 import           Prelude hiding ((<*), (*>))
 
 import           Control.Applicative (liftA2)
-import           Data.Bifunctor (Bifunctor(..))
+--import           Data.Bifunctor (Bifunctor(..))
 import           Data.Data (Data) -- toConstr
-import           Data.List (delete, findIndex, sort, intercalate)
+import           Data.List (delete, findIndex, sort)
 import qualified Data.Map.Strict as Map
-import           Data.Maybe (isJust, fromJust, catMaybes, fromMaybe, mapMaybe)
-import           Data.Ratio (numerator, denominator)
+import           Data.Maybe (isJust, fromJust, catMaybes, mapMaybe)
 import           Data.Typeable (Typeable)
+import           Control.DeepSeq (NFData(rnf))
+import           Data.Word (Word8)
 
 ------------------------------------------------------------------------
 -- * Types
 
 -- | Time is rational
-type Time = Rational
+type Time = Rational 
 
 -- | The 'sam' (start of cycle) for the given time value
 sam :: Time -> Time
@@ -45,8 +46,9 @@ data ArcF a = Arc
 
 type Arc = ArcF Time
 
-instance {-# OVERLAPPING #-} Show Arc where
-  show (Arc s e) = prettyRat s ++ ">" ++ prettyRat e
+instance NFData a => 
+  NFData (ArcF a) where 
+    rnf (Arc s e) = rnf s `seq` rnf e
 
 instance Num a => Num (ArcF a) where
   negate      = fmap negate
@@ -80,6 +82,11 @@ subArc a@(Arc s e) b@(Arc s' e')
   | otherwise = Nothing
   where (Arc s'' e'') = sect a b
 
+subMaybeArc :: Maybe Arc -> Maybe Arc -> Maybe (Maybe Arc)
+subMaybeArc (Just a) (Just b) = do sa <- subArc a b
+                                   return $ Just sa
+subMaybeArc _ _ = Just Nothing
+
 instance Applicative ArcF where
   pure t = Arc t t
   (<*>) (Arc sf ef) (Arc sx ex) = Arc (sf sx) (ef ex)
@@ -125,27 +132,52 @@ mapCycle f (Arc s e) = Arc (sam' + f (s - sam')) (sam' + f (e - sam'))
 isIn :: Arc -> Time -> Bool
 isIn (Arc s e) t = t >= s && t < e
 
--- | An event is a value that's active during a timespan
--- The part should be equal to or fit inside the
--- whole
+data Context = Context {contextPosition :: [((Int, Int), (Int, Int))]}
+  deriving (Eq, Ord)
+
+instance NFData Context where 
+    rnf (Context c) = rnf c
+
+combineContexts :: [Context] -> Context
+combineContexts = Context . concatMap contextPosition
+
+setContext :: Context -> Pattern a -> Pattern a
+setContext c pat = withEvents (map (\e -> e {context = c})) pat
+
+withContext :: (Context -> Context) -> Pattern a -> Pattern a
+withContext f pat = withEvents (map (\e -> e {context = f $ context e})) pat
+
+deltaContext :: Int -> Int -> Pattern a -> Pattern a
+deltaContext column line pat = withEvents (map (\e -> e {context = f $ context e})) pat
+  where f :: Context -> Context
+        f (Context xs) = Context $ map (\((bx,by), (ex,ey)) -> ((bx+column,by+line), (ex+column,ey+line))) xs
+
+-- | An event is a value that's active during a timespan. If a whole
+-- is present, the part should be equal to or fit inside it.
 data EventF a b = Event
-  { whole :: a
+  { context :: Context
+  , whole :: Maybe a
   , part :: a
   , value :: b
   } deriving (Eq, Ord, Functor)
 
 type Event a = EventF (ArcF Time) a
 
-instance Bifunctor EventF where
-  bimap f g (Event w p e) = Event (f w) (f p) (g e)
+instance (NFData a, NFData b) => 
+  NFData (EventF a b) where 
+    rnf (Event c w p v) = rnf c `seq` rnf w `seq` rnf p `seq` rnf v
 
-instance {-# OVERLAPPING #-} Show a => Show (Event a) where
-  show (Event (Arc ws we) a@(Arc ps pe) e) =
-    h ++ "(" ++ show a ++ ")" ++ t ++ "|" ++ show e
-    where h | ws == ps = ""
-            | otherwise = prettyRat ws ++ "-"
-          t | we == pe = ""
-            | otherwise = "-" ++ prettyRat we
+{-instance Bifunctor EventF where
+  bimap f g (Event w p e) = Event (f w) (f p) (g e)
+-}
+
+
+isAnalog :: Event a -> Bool
+isAnalog (Event {whole = Nothing}) = True
+isAnalog _ = False
+
+isDigital :: Event a -> Bool
+isDigital = not . isAnalog
 
 -- | `True` if an `Event`'s starts is within given `Arc`
 onsetIn :: Arc -> Event a -> Bool
@@ -164,7 +196,7 @@ defragParts (e:es) | isJust i = defraged : defragParts (delete e' es)
                    | otherwise = e : defragParts es
   where i = findIndex (isAdjacent e) es
         e' = es !! fromJust i
-        defraged = Event (whole e) u (value e)
+        defraged = Event (context e) (whole e) u (value e)
         u = hull (part e) (part e')
 
 -- | Returns 'True' if the two given events are adjacent parts of the same whole
@@ -176,13 +208,17 @@ isAdjacent e e' = (whole e == whole e')
                       (stop (part e') == start (part e))
                      )
 
+wholeOrPart :: Event a -> Arc
+wholeOrPart (Event {whole = Just a}) = a
+wholeOrPart e = part e
+
 -- | Get the onset of an event's 'whole'
 wholeStart :: Event a -> Time
-wholeStart = start . whole
+wholeStart = start . wholeOrPart
 
 -- | Get the offset of an event's 'whole'
 wholeStop :: Event a -> Time
-wholeStop = stop . whole
+wholeStop = stop . wholeOrPart
 
 -- | Get the onset of an event's 'whole'
 eventPartStart :: Event a -> Time
@@ -200,10 +236,13 @@ eventValue :: Event a -> a
 eventValue = value
 
 eventHasOnset :: Event a -> Bool
-eventHasOnset e = start (whole e) == start (part e)
+eventHasOnset e | isAnalog e = False
+                | otherwise = start (fromJust $ whole e) == start (part e)
 
+-- TODO - Is this used anywhere? Just tests, it seems
+-- TODO - support 'context' field
 toEvent :: (((Time, Time), (Time, Time)), a) -> Event a
-toEvent (((ws, we), (ps, pe)), v) = Event (Arc ws we) (Arc ps pe) v
+toEvent (((ws, we), (ps, pe)), v) = Event (Context []) (Just $ Arc ws we) (Arc ps pe) v
 
 -- | an Arc and some named control values
 data State = State {arc :: Arc,
@@ -213,24 +252,28 @@ data State = State {arc :: Arc,
 -- | A function that represents events taking place over time
 type Query a = (State -> [Event a])
 
--- | Also known as Continuous vs Discrete/Amorphous vs Pulsating etc.
-data Nature = Analog | Digital
-            deriving (Eq, Show)
-
--- | A datatype that's basically a query, plus a hint about whether its events
--- are Analogue or Digital by nature
-data Pattern a = Pattern {nature :: Nature, query :: Query a}
+-- | A datatype that's basically a query
+data Pattern a = Pattern {query :: Query a}
 
 data Value = VS { svalue :: String }
            | VF { fvalue :: Double }
            | VR { rvalue :: Rational }
            | VI { ivalue :: Int }
            | VB { bvalue :: Bool }
+           | VX { xvalue :: [Word8] } -- Used for OSC 'blobs'
            deriving (Typeable,Data)
 
 class Valuable a where
   toValue :: a -> Value
 
+instance NFData Value where 
+  rnf (VS s) = rnf s 
+  rnf (VF f) = rnf f 
+  rnf (VR r) = rnf r 
+  rnf (VI i) = rnf i 
+  rnf (VB b) = rnf b
+  rnf (VX xs) = rnf xs
+
 instance Valuable String where
   toValue = VS
 instance Valuable Double where
@@ -241,14 +284,16 @@ instance Valuable Int where
   toValue a = VI a
 instance Valuable Bool where
   toValue a = VB a
+instance Valuable [Word8] where
+  toValue a = VX a
 
 instance Eq Value where
   (VS x) == (VS y) = x == y
   (VB x) == (VB y) = x == y
-
   (VF x) == (VF y) = x == y
   (VI x) == (VI y) = x == y
   (VR x) == (VR y) = x == y
+  (VX x) == (VX y) = x == y
   
   (VF x) == (VI y) = x == (fromIntegral y)
   (VI y) == (VF x) = x == (fromIntegral y)
@@ -258,7 +303,6 @@ instance Eq Value where
   (VI x) == (VR y) = (toRational x) == y
   (VR y) == (VI x) = (toRational x) == y
 
-
   _ == _ = False
   
 instance Ord Value where
@@ -267,10 +311,13 @@ instance Ord Value where
   compare (VF x) (VF y) = compare x y
   compare (VI x) (VI y) = compare x y
   compare (VR x) (VR y) = compare x y
+  compare (VX x) (VX y) = compare x y
   compare (VS _) _ = LT
   compare _ (VS _) = GT
   compare (VB _) _ = LT
   compare _ (VB _) = GT
+  compare (VX _) _ = LT
+  compare _ (VX _) = GT
   compare (VF x) (VI y) = compare x (fromIntegral y)
   compare (VI x) (VF y) = compare (fromIntegral x) y
 
@@ -287,88 +334,75 @@ type ControlPattern = Pattern ControlMap
 ------------------------------------------------------------------------
 -- * Instances
 
+instance NFData a => 
+  NFData (Pattern a) where 
+    rnf (Pattern q) = rnf $ \s -> q s
+
 instance Functor Pattern where
   -- | apply a function to all the values in a pattern
   fmap f p = p {query = fmap (fmap f) . query p}
 
-instance Applicative Pattern where
-  -- | Repeat the given value once per cycle, forever
-  pure v = Pattern Digital $ \(State a _) ->
-    map (\a' -> Event a' (sect a a') v) $ cycleArcsInArc a
-
-  (<*>) pf@(Pattern Digital _) px@(Pattern Digital _) = Pattern Digital q
+applyPatToPat :: (Maybe Arc -> Maybe Arc -> Maybe (Maybe Arc)) -> Pattern (a -> b) -> Pattern a -> Pattern b
+applyPatToPat combineWholes pf px = Pattern q
     where q st = catMaybes $ concatMap match $ query pf st
             where
-              match (Event fWhole fPart f) =
+              match (ef@(Event (Context c) _ fPart f)) =
                 map
-                (\(Event xWhole xPart x) ->
-                  do whole' <- subArc xWhole fWhole
+                (\ex@(Event (Context c') _ xPart x) ->
+                  do whole' <- combineWholes (whole ef) (whole ex)
                      part' <- subArc fPart xPart
-                     return (Event whole' part' (f x))
+                     return (Event (Context $ c ++ c') whole' part' (f x))
                 )
-                (query px $ st {arc = fPart})
-  (<*>) pf@(Pattern Digital _) px@(Pattern Analog _) = Pattern Digital q
-    where q st = concatMap match $ query pf st
-            where
-              match (Event fWhole fPart f) =
-                map
-                (Event fWhole fPart . f . value)
-                (query px $ st {arc = pure (start fPart)})
+                (query px $ st {arc = (wholeOrPart ef)})
 
-  (<*>) pf@(Pattern Analog _) px@(Pattern Digital _) = Pattern Digital q
-    where q st = concatMap match $ query px st
-            where
-              match (Event xWhole xPart x) =
-                map
-                (\e -> Event xWhole xPart (value e x))
-                (query pf st {arc = pure (start xPart)})
+instance Applicative Pattern where
+  -- | Repeat the given value once per cycle, forever
+  pure v = Pattern $ \(State a _) ->
+    map (\a' -> Event (Context []) (Just a') (sect a a') v) $ cycleArcsInArc a
 
-  (<*>) pf px = Pattern Analog q
-    where q st = concatMap match $ query pf st
-            where
-              match ef =
-                map
-                (Event (arc st) (arc st) . value ef . value)
-                (query px st)
+  (<*>) = applyPatToPatBoth
 
--- | Like <*>, but the structure only comes from the left
+applyPatToPatBoth :: Pattern (a -> b) -> Pattern a -> Pattern b
+applyPatToPatBoth pf px = Pattern q
+    where q st = catMaybes $ (concatMap match $ query pf st) ++ (concatMap matchX $ query (filterAnalog px) st)
+            where
+              -- match analog events from pf with all events from px
+              match ef@(Event _ Nothing fPart _)   = map (withFX ef) (query px $ st {arc = fPart}) -- analog
+              -- match digital events from pf with digital events from px
+              match ef@(Event _ (Just fWhole) _ _) = map (withFX ef) (query (filterDigital px) $ st {arc = fWhole}) -- digital
+              -- match analog events from px (constrained above) with digital events from px
+              matchX ex@(Event _ Nothing fPart _)  = map (\ef -> withFX ef ex) (query (filterDigital pf) $ st {arc = fPart}) -- digital
+              matchX _ = error "can't happen"
+              withFX ef ex = do whole' <- subMaybeArc (whole ef) (whole ex)
+                                part' <- subArc (part ef) (part ex)
+                                return (Event (combineContexts [context ef, context ex]) whole' part' (value ef $ value ex))
+
+applyPatToPatLeft :: Pattern (a -> b) -> Pattern a -> Pattern b
+applyPatToPatLeft pf px = Pattern q
+    where q st = catMaybes $ (concatMap match $ query pf st)
+            where
+              match ef = map (withFX ef) (query px $ st {arc = wholeOrPart ef})
+              withFX ef ex = do let whole' = whole ef
+                                part' <- subArc (part ef) (part ex)
+                                return (Event (combineContexts [context ef, context ex]) whole' part' (value ef $ value ex))
+
+applyPatToPatRight :: Pattern (a -> b) -> Pattern a -> Pattern b
+applyPatToPatRight pf px = Pattern q
+    where q st = catMaybes $ (concatMap match $ query px st)
+            where
+              match ex = map (\ef -> withFX ef ex) (query pf $ st {arc = wholeOrPart ex})
+              withFX ef ex = do let whole' = whole ex
+                                part' <- subArc (part ef) (part ex)
+                                return (Event (combineContexts [context ef, context ex]) whole' part' (value ef $ value ex))
+
+
+-- | Like <*>, but the 'wholes' come from the left
 (<*) :: Pattern (a -> b) -> Pattern a -> Pattern b
-(<*) pf@(Pattern Analog _) px@(Pattern Analog _) = Pattern Analog q
-  where q st = concatMap match $ query pf st
-          where
-            match (Event fWhole fPart f) =
-              map
-              (Event fWhole fPart . f . value) $
-              query px st -- for continuous events, use the original query
+(<*) = applyPatToPatLeft
 
--- If one of the patterns is digital, treat both as digital.. (TODO - needs extra thought)
-(<*) pf px = Pattern Digital q
-  where q st = concatMap match $ query pf st
-         where
-            match (Event fWhole fPart f) =
-              map
-              (Event fWhole fPart . f . value) $
-              query px $ st {arc = xQuery fWhole}
-            xQuery (Arc s _) = pure s -- for discrete events, match with the onset
-
--- | Like <*>, but the structure only comes from the right
+-- | Like <*>, but the 'wholes' come from the right
 (*>) :: Pattern (a -> b) -> Pattern a -> Pattern b
-(*>) pf@(Pattern Analog _) px@(Pattern Analog _) = Pattern Analog q
-  where q st = concatMap match $ query px st
-          where
-            match (Event xWhole xPart x) =
-              map
-              (\e -> Event xWhole xPart (value e x)) $
-              query pf st -- for continuous events, use the original query
-
-(*>) pf px = Pattern Digital q
-  where q st = concatMap match $ query px st
-         where
-            match (Event xWhole xPart x) =
-              map
-              (\e -> Event xWhole xPart (value e x)) $
-              query pf $ fQuery xWhole
-            fQuery (Arc s _) = st {arc = pure s} -- for discrete events, match with the onset
+(*>) = applyPatToPatRight
 
 infixl 4 <*, *>
 
@@ -389,42 +423,42 @@ instance Monad Pattern where
 unwrap :: Pattern (Pattern a) -> Pattern a
 unwrap pp = pp {query = q}
   where q st = concatMap
-          (\(Event w p v) ->
-             mapMaybe (munge w p) $ query v st {arc = p})
+          (\(Event c w p v) ->
+             mapMaybe (munge c w p) $ query v st {arc = p})
           (query pp st)
-        munge ow op (Event iw ip v') =
+        munge oc ow op (Event ic iw ip v') =
           do
-            w' <- subArc ow iw
+            w' <- subMaybeArc ow iw
             p' <- subArc op ip
-            return (Event w' p' v')
+            return (Event (combineContexts [ic, oc]) w' p' v')
 
 -- | Turns a pattern of patterns into a single pattern. Like @unwrap@,
 -- but structure only comes from the inner pattern.
 innerJoin :: Pattern (Pattern a) -> Pattern a
 innerJoin pp = pp {query = q}
   where q st = concatMap
-               (\(Event _ p v) -> mapMaybe munge $ query v st {arc = p}
+               (\(Event oc _ op v) -> mapMaybe (munge oc) $ query v st {arc = op}
           )
           (query pp st)
-          where munge (Event iw ip v) =
+          where munge oc (Event ic iw ip v) =
                   do
                     p <- subArc (arc st) ip
                     p' <- subArc p (arc st)
-                    return (Event iw p' v)
+                    return (Event (combineContexts [ic, oc]) iw p' v)
 
 -- | Turns a pattern of patterns into a single pattern. Like @unwrap@,
 -- but structure only comes from the outer pattern.
 outerJoin :: Pattern (Pattern a) -> Pattern a
 outerJoin pp = pp {query = q}
   where q st = concatMap
-          (\(Event w p v) ->
-             mapMaybe (munge w p) $ query v st {arc = pure (start w)}
+          (\e ->
+             mapMaybe (munge (context e) (whole e) (part e)) $ query (value e) st {arc = pure (start $ wholeOrPart e)}
           )
           (query pp st)
-          where munge ow op (Event _ _ v') =
+          where munge oc ow op (Event ic _ _ v') =
                   do
                     p' <- subArc (arc st) op
-                    return (Event ow p' v')
+                    return (Event (combineContexts [oc, ic]) ow p' v')
 
 -- | Like @unwrap@, but cycles of the inner patterns are compressed to fit the
 -- timespan of the outer whole (or the original query if it's a continuous pattern?)
@@ -432,14 +466,14 @@ outerJoin pp = pp {query = q}
 squeezeJoin :: Pattern (Pattern a) -> Pattern a
 squeezeJoin pp = pp {query = q}
   where q st = concatMap
-          (\(Event w p v) ->
-             mapMaybe (munge w p) $ query (compressArc (cycleArc w) v) st {arc = p}
+          (\e@(Event c w p v) ->
+             mapMaybe (munge c w p) $ query (compressArc (cycleArc $ wholeOrPart e) v) st {arc = p}
           )
           (query pp st)
-        munge oWhole oPart (Event iWhole iPart v) =
-          do w' <- subArc oWhole iWhole
+        munge oContext oWhole oPart (Event iContext iWhole iPart v) =
+          do w' <- subMaybeArc oWhole iWhole
              p' <- subArc oPart iPart
-             return (Event w' p' v)
+             return (Event (combineContexts [iContext, oContext]) w' p' v)
 
 noOv :: String -> a
 noOv meth = error $ meth ++ ": not supported for patterns"
@@ -458,7 +492,7 @@ instance TolerantEq ControlMap where
   a ~== b = Map.differenceWith (\a' b' -> if a' ~== b' then Nothing else Just a') a b == Map.empty
 
 instance TolerantEq (Event ControlMap) where
-  (Event w p x) ~== (Event w' p' x') = w == w' && p == p' && x ~== x'
+  (Event _ w p x) ~== (Event _ w' p' x') = w == w' && p == p' && x ~== x'
 
 instance TolerantEq a => TolerantEq [a] where
   as ~== bs = (length as == length bs) && all (uncurry (~==)) (zip as bs)
@@ -558,91 +592,15 @@ instance Fractional ControlMap where
   recip        = fmap (applyFIS recip id id)
   fromRational = Map.singleton "speed" . VF . fromRational
 
-showPattern :: Show a => Arc -> Pattern a -> String
-showPattern a p = intercalate "\n" $ map show $ queryArc p a
-
-instance (Show a) => Show (Pattern a) where
-  show = showPattern (Arc 0 1)
-
-instance Show Value where
-  show (VS s) = ('"':s) ++ "\""
-  show (VI i) = show i
-  show (VF f) = show f ++ "f"
-  show (VR r) = show r ++ "r"
-  show (VB b) = show b
-
-instance {-# OVERLAPPING #-} Show ControlMap where
-  show m = intercalate ", " $ map (\(name, v) -> name ++ ": " ++ show v) $ Map.toList m
-
-prettyRat :: Rational -> String
-prettyRat r | unit == 0 && frac > 0 = showFrac (numerator frac) (denominator frac)
-            | otherwise =  show unit ++ showFrac (numerator frac) (denominator frac)
-  where unit = floor r :: Int
-        frac = r - toRational unit
-
-showFrac :: Integer -> Integer -> String
-showFrac 0 _ = ""
-showFrac 1 2 = "½"
-showFrac 1 3 = "⅓"
-showFrac 2 3 = "⅔"
-showFrac 1 4 = "¼"
-showFrac 3 4 = "¾"
-showFrac 1 5 = "⅕"
-showFrac 2 5 = "⅖"
-showFrac 3 5 = "⅗"
-showFrac 4 5 = "⅘"
-showFrac 1 6 = "⅙"
-showFrac 5 6 = "⅚"
-showFrac 1 7 = "⅐"
-showFrac 1 8 = "⅛"
-showFrac 3 8 = "⅜"
-showFrac 5 8 = "⅝"
-showFrac 7 8 = "⅞"
-showFrac 1 9 = "⅑"
-showFrac 1 10 = "⅒"
-
-showFrac n d = fromMaybe plain $ do n' <- up n
-                                    d' <- down d
-                                    return $ n' ++ d'
-  where plain = " " ++ show n ++ "/" ++ show d
-        up 1 = Just "¹"
-        up 2 = Just "²"
-        up 3 = Just "³"
-        up 4 = Just "⁴"
-        up 5 = Just "⁵"
-        up 6 = Just "⁶"
-        up 7 = Just "⁷"
-        up 8 = Just "⁸"
-        up 9 = Just "⁹"
-        up 0 = Just "⁰"
-        up _ = Nothing
-        down 1 = Just "₁"
-        down 2 = Just "₂"
-        down 3 = Just "₃"
-        down 4 = Just "₄"
-        down 5 = Just "₅"
-        down 6 = Just "₆"
-        down 7 = Just "₇"
-        down 8 = Just "₈"
-        down 9 = Just "₉"
-        down 0 = Just "₀"
-        down _ = Nothing
-
 ------------------------------------------------------------------------
 -- * Internal functions
 
 empty :: Pattern a
-empty = Pattern {nature = Digital, query = const []}
+empty = Pattern {query = const []}
 
 queryArc :: Pattern a -> Arc -> [Event a]
 queryArc p a = query p $ State a Map.empty 
 
-isDigital :: Pattern a -> Bool
-isDigital = (== Digital) . nature
-
-isAnalog :: Pattern a -> Bool
-isAnalog = not . isDigital
-
 -- | Splits queries that span cycles. For example `query p (0.5, 1.5)` would be
 -- turned into two queries, `(0.5,1)` and `(1,1.5)`, and the results
 -- combined. Being able to assume queries don't span cycles often
@@ -653,7 +611,7 @@ splitQueries p = p {query = \st -> concatMap (\a -> query p st {arc = a}) $ arcC
 -- | Apply a function to the arcs/timespans (both whole and parts) of the result
 withResultArc :: (Arc -> Arc) -> Pattern a -> Pattern a
 withResultArc f pat = pat
-  { query = map (\(Event w p e) -> Event (f w) (f p) e) . query pat}
+  { query = map (\(Event c w p e) -> Event c (f <$> w) (f p) e) . query pat}
 
 -- | Apply a function to the time (both start and end of the timespans
 -- of both whole and parts) of the result
@@ -681,7 +639,7 @@ withEvents f p = p {query = f . query p}
 -- | @withPart f p@ returns a new @Pattern@ with function @f@ applied
 -- to the part.
 withPart :: (Arc -> Arc) -> Pattern a -> Pattern a
-withPart f = withEvent (\(Event w p v) -> Event w (f p) v)
+withPart f = withEvent (\(Event c w p v) -> Event c w (f p) v)
 
 -- | Apply one of three functions to a Value, depending on its type
 applyFIS :: (Double -> Double) -> (Int -> Int) -> (String -> String) -> Value -> Value
@@ -725,6 +683,10 @@ getR (VF x) = Just $ toRational x
 getR (VI x) = Just $ toRational x
 getR _  = Nothing
 
+getBlob :: Value -> Maybe [Word8]
+getBlob (VX xs) = Just xs
+getBlob _  = Nothing
+
 compressArc :: Arc -> Pattern a -> Pattern a
 compressArc (Arc s e) p | s > e = empty
                         | s > 1 || e > 1 = empty
@@ -770,6 +732,18 @@ filterJust p = fromJust <$> filterValues isJust p
 filterWhen :: (Time -> Bool) -> Pattern a -> Pattern a
 filterWhen test p = p {query = filter (test . wholeStart) . query p}
 
+filterOnsets :: Pattern a -> Pattern a
+filterOnsets p = p {query = filter (\e -> eventPartStart e == wholeStart e) . query (filterDigital p)}
+
+filterEvents :: (Event a -> Bool) -> Pattern a -> Pattern a
+filterEvents f p = p {query = filter f . query p}
+
+filterDigital :: Pattern a -> Pattern a
+filterDigital = filterEvents isDigital
+
+filterAnalog :: Pattern a -> Pattern a
+filterAnalog = filterEvents isAnalog
+
 playFor :: Time -> Time -> Pattern a -> Pattern a
 playFor s e = filterWhen (\t -> (t >= s) && (t < e))
 
@@ -793,7 +767,8 @@ matchManyToOne :: (b -> a -> Bool) -> Pattern a -> Pattern b -> Pattern (Bool, b
 matchManyToOne f pa pb = pa {query = q}
   where q st = map match $ query pb st
           where
-            match (Event xWhole xPart x) =
-              Event xWhole xPart (any (f x) (as $ start xWhole), x)
-            as s = map value $ query pa $ fQuery s
+            match (ex@(Event xContext xWhole xPart x)) =
+              Event (combineContexts $ xContext:(map context as')) xWhole xPart (any (f x) (map value $ as'), x)
+                where as' = as $ start $ wholeOrPart ex
+            as s = query pa $ fQuery s
             fQuery s = st {arc = Arc s s}
