@@ -1,4 +1,4 @@
-{-# LANGUAGE ConstraintKinds, GeneralizedNewtypeDeriving, FlexibleContexts, ScopedTypeVariables #-}
+{-# LANGUAGE ConstraintKinds, GeneralizedNewtypeDeriving, FlexibleContexts, ScopedTypeVariables, BangPatterns #-}
 {-# OPTIONS_GHC -fno-warn-missing-fields #-}
 
 module Sound.Tidal.Stream where
@@ -23,6 +23,9 @@ import           Sound.Tidal.Core (stack, silence)
 import           Sound.Tidal.Pattern
 import qualified Sound.Tidal.Tempo as T
 -- import qualified Sound.OSC.Datum as O
+import           Data.List (sortOn)
+import           System.Random (getStdRandom, randomR)
+import           Sound.Tidal.Show ()
 
 data TimeStamp = BundleStamp | MessageStamp | NoStamp
  deriving (Eq, Show)
@@ -103,7 +106,8 @@ dirtTarget = OSCTarget {oName = "Dirt",
                                        ("attack", Just $ VF (-1)),
                                        ("hold", Just $ VF 0),
                                        ("release", Just $ VF (-1)),
-                                       ("orbit", Just $ VI 0)
+                                       ("orbit", Just $ VI 0),
+                                       ("id", Just $ VI 0)
                                       ],
                          oLatency = 0.02,
                          oPreamble = [],
@@ -138,26 +142,61 @@ toDatum (VS x) = O.string x
 toDatum (VR x) = O.float $ ((fromRational x) :: Double)
 toDatum (VB True) = O.int32 (1 :: Int)
 toDatum (VB False) = O.int32 (0 :: Int)
+toDatum (VX xs) = O.Blob $ O.blob_pack xs
 
 toData :: OSCTarget -> Event ControlMap -> Maybe [O.Datum]
 toData target e
   | isJust (oShape target) = fmap (fmap toDatum) $ sequence $ map (\(n,v) -> Map.lookup n (value e) <|> v) (fromJust $ oShape target)
   | otherwise = Just $ concatMap (\(n,v) -> [O.string n, toDatum v]) $ Map.toList $ value e
 
-toMessage :: Double -> OSCTarget -> T.Tempo -> Event (Map.Map String Value) -> Maybe O.Message
-toMessage t target tempo e = do vs <- toData target addExtra
-                                return $ O.Message (oPath target) $ oPreamble target ++ vs
-  where on = sched tempo $ start $ whole e
-        off = sched tempo $ stop $ whole e
+substitutePath :: String -> ControlMap -> String
+substitutePath path cm = parse path
+  where parse [] = []
+        parse ('{':xs) = parseWord xs
+        parse (x:xs) = x:(parse xs)
+        parseWord xs | b == [] = getString cm a
+                     | otherwise = getString cm a ++ parse (tail b)
+          where (a,b) = break (== '}') xs
+
+getString :: ControlMap -> String -> String
+getString cm s = fromMaybe "" $ do v <- Map.lookup s cm
+                                   return $ simpleShow v
+                                    where simpleShow :: Value -> String
+                                          simpleShow (VS str) = str
+                                          simpleShow (VI i) = show i
+                                          simpleShow (VF f) = show f
+                                          simpleShow (VR r) = show r
+                                          simpleShow (VB b) = show b
+                                          simpleShow (VX xs) = show xs
+
+toMessage :: Config -> Double -> OSCTarget -> T.Tempo -> Event (Map.Map String Value) -> Maybe O.Message
+toMessage config t target tempo e = do vs <- toData target addExtra
+                                       return $ O.Message (substitutePath (oPath target) (value e)) $ oPreamble target ++ vs
+  where on = sched tempo $ start $ wholeOrPart e
+        off = sched tempo $ stop $ wholeOrPart e
+        cm = value e
+        identifier = ((if (start $ wholeOrPart e) == (start $ part e) then "X" else ">")
+                      ++ show (start $ wholeOrPart e)
+                      ++ "-"
+                      ++ show (stop $ wholeOrPart e)
+                      ++ "-"
+                      ++ getString cm "n"
+                      ++ "-"
+                      ++ getString cm "note"
+                      ++ "-"
+                      ++ getString cm "s"
+                     )
         delta = off - on
         messageStamp = oTimestamp target == MessageStamp
         -- If there is already cps in the event, the union will preserve that.
         addExtra = (\v -> (Map.union v $ Map.fromList (extra messageStamp)
                           )) <$> e
-        extra False = [("cps", (VF $ T.cps tempo)),
-                       ("delta", VF delta),
-                       ("cycle", VF (fromRational $ start $ whole e))
-                      ]
+        addIdentifier | cSendParts config = (("id", VS identifier):)
+                      | otherwise = id
+        extra False = addIdentifier [("cps", (VF $ T.cps tempo)),
+                                     ("delta", VF delta),
+                                     ("cycle", VF (fromRational $ start $ wholeOrPart e))
+                                    ]
         extra True = timestamp ++ (extra False)
         timestamp = [("sec", VI sec),
                      ("usec", VI usec)
@@ -178,23 +217,42 @@ onTick :: Config -> MVar StateMap -> MVar ControlPattern -> [Cx] -> MVar T.Tempo
 onTick config sMapMV pMV cxs tempoMV st =
   do p <- readMVar pMV
      sMap <- readMVar sMapMV
-     tempo <- readMVar tempoMV
-     now <- O.time
-     let sMap' = Map.insert "_cps" (pure $ VF $ T.cps tempo) sMap
-         es = filter eventHasOnset $ query p (State {arc = T.nowArc st, controls = sMap'})
-         on e = (sched tempo $ start $ whole e) + eventNudge e
+     tempo <- takeMVar tempoMV
+     let frameEnd = snd $ T.nowTimespan st
+         sMap' = Map.insert "_cps" (pure $ VF $ T.cps tempo) sMap
+         es = sortOn (start . part) $ filterOns $ query p (State {arc = T.nowArc st, controls = sMap'})
+         filterOns | cSendParts config = id
+                   | otherwise = filter eventHasOnset
+           -- there should always be a whole (due to the eventHasOnset filter)
+         on e tempo'' = (sched tempo'' $ start $ wholeOrPart e)
          eventNudge e = fromJust $ getF $ fromMaybe (VF 0) $ Map.lookup "nudge" $ value e
-         messages target = catMaybes $ map (\e -> do m <- toMessage (on e + latency target) target tempo e
-                                                     return $ (on e, m)
-                                           ) es
-         cpsChanges = map (\e -> (on e - now, Map.lookup "cps" $ value e)) es
+         processCps :: T.Tempo -> [Event ControlMap] -> ([(T.Tempo, Event ControlMap)], T.Tempo)
+         processCps t [] = ([], t)
+         -- If an event has a tempo change, that affects the following
+         -- events..
+         processCps t (e:evs) = (((t', e):es'), t'')
+           where cps' = do x <- Map.lookup "cps" $ value e
+                           getF x
+                 t' = (maybe t (\newCps -> T.changeTempo' t newCps (eventPartStart e)) cps')
+                 (es', t'') = processCps t' evs
          latency target = oLatency target + cFrameTimespan config + T.nudged tempo
-     mapM_ (\(Cx target udp) -> E.catch (mapM_ (send target (latency target) udp) (messages target))
-                       (\(_ ::E.SomeException)
-                        -> putStrLn $ "Failed to send. Is the '" ++ oName target ++ "' target running?"
+         (tes, tempo') = processCps tempo es
+     mapM_ (\(Cx target udp) -> (do let ms = catMaybes $ map (\(t, e) -> do let nudge = eventNudge e
+                                                                            let onset = on e t
+                                                                            m <- toMessage config (onset + nudge + latency target) target tempo e
+                                                                            -- drop events that have gone out of frame (due to tempo
+                                                                            -- changes during the frame)
+                                                                            if (onset < frameEnd)
+                                                                              then Just (onset + nudge, m)
+                                                                              else  Nothing
+                                                             ) tes
+                                    E.catch (mapM_ (send target (latency target) udp) ms)
+                                )
+                       (\(e ::E.SomeException)
+                        -> putStrLn $ "Failed to send. Is the '" ++ oName target ++ "' target running? " ++ show e
                        )
            ) cxs
-     mapM_ (doCps tempoMV) cpsChanges
+     putMVar tempoMV tempo'
      return ()
 
 send :: O.Transport t => OSCTarget -> Double -> t -> (Double, O.Message) -> IO ()
@@ -234,7 +292,7 @@ streamList s = do pMap <- readMVar (sPMapMV s)
 
 -- Evaluation of pat is forced so exceptions are picked up here, before replacing the existing pattern.
 streamReplace :: Show a => Stream -> a -> ControlPattern -> IO ()
-streamReplace s k pat
+streamReplace s k !pat
   = E.catch (do let x = queryArc pat (Arc 0 0)
                 tempo <- readMVar $ sTempoMV s
                 input <- takeMVar $ sInput s
@@ -271,7 +329,11 @@ streamUnsolo :: Show a => Stream -> a -> IO ()
 streamUnsolo s k = withPatId s (show k) (\x -> x {solo = False})
 
 streamOnce :: Stream -> ControlPattern -> IO ()
-streamOnce st p
+streamOnce st p = do i <- getStdRandom $ randomR (0, 8192)
+                     streamFirst st $ rotL (toRational (i :: Int)) p
+
+streamFirst :: Stream -> ControlPattern -> IO ()
+streamFirst st p
   = do sMap <- readMVar (sInput st)
        tempo <- readMVar (sTempoMV st)
        now <- O.time
@@ -286,17 +348,20 @@ streamOnce st p
                                                        controls = sMap'
                                                       }
                                                )
-           at e = sched fakeTempo $ start $ whole e
-           on e = sched tempo $ start $ whole e
+           -- there should always be a whole (due to the eventHasOnset filter)
+           at e = sched fakeTempo $ start $ wholeOrPart e
+           -- there should always be a whole (due to the eventHasOnset filter)
+           on e = sched tempo $ start $ wholeOrPart e
            cpsChanges = map (\e -> (on e - now, Map.lookup "cps" $ value e)) es
+           config = sConfig st
            messages target =
-             catMaybes $ map (\e -> do m <- toMessage (at e + (oLatency target)) target fakeTempo e
+             catMaybes $ map (\e -> do m <- toMessage config (at e + (oLatency target)) target fakeTempo e
                                        return $ (at e, m)
                              ) es
        mapM_ (\(Cx target udp) ->
                  E.catch (mapM_ (send target (oLatency target) udp) (messages target))
-                 (\(_ ::E.SomeException)
-                   -> putStrLn $ "Failed to send. Is the '" ++ oName target ++ "' target running?"
+                 (\(e ::E.SomeException)
+                   -> putStrLn $ "Failed to send. Is the '" ++ oName target ++ "' target running? " ++ show e
                  )
              ) (sCxs st)
        mapM_ (doCps $ sTempoMV st) cpsChanges
