@@ -4,14 +4,16 @@ module Sound.Tidal.UI where
 
 import           Prelude hiding ((<*), (*>))
 
-import           Data.Char (digitToInt, isDigit)
+import           Data.Char (digitToInt, isDigit, ord)
+import           Data.Bits (testBit, Bits)
 -- import           System.Random (randoms, mkStdGen)
 import           System.Random.MWC
 import           Control.Monad.ST
+import           Control.Monad.Primitive (PrimState, PrimMonad)
 import qualified Data.Vector as V
 import           Data.Word (Word32)
 import           Data.Ratio ((%),numerator,denominator)
-import           Data.List (sort, sortOn, findIndices, elemIndex, groupBy, transpose, intercalate)
+import           Data.List (sort, sortOn, findIndices, elemIndex, groupBy, transpose, intercalate, findIndex)
 import           Data.Maybe (isJust, fromJust, fromMaybe, mapMaybe)
 import qualified Data.Text as T
 import qualified Data.Map.Strict as Map
@@ -27,7 +29,7 @@ import           Sound.Tidal.Utils
 -- * UI
 
 -- | Randomisation
-
+timeToSeed :: (PrimMonad m, Real a) => a -> m (Gen (PrimState m))
 timeToSeed x = do
   let x' = toRational (x*x) / 1000000
   let n' = fromIntegral $ numerator x'
@@ -78,7 +80,7 @@ jux (# ((1024 <~) $ gain rand)) $ sound "sn sn ~ sn" # gain rand
 @
 -}
 rand :: Fractional a => Pattern a
-rand = Pattern Analog (\(State a@(Arc s e) _) -> [Event a a (realToFrac $ timeToRand $ (e + s)/2)])
+rand = Pattern (\(State a@(Arc s e) _) -> [Event (Context []) Nothing a (realToFrac $ timeToRand $ (e + s)/2)])
 
 {- | Just like `rand` but for whole numbers, `irand n` generates a pattern of (pseudo-) random whole numbers between `0` to `n-1` inclusive. Notably used to pick a random
 samples from a folder:
@@ -195,16 +197,20 @@ degradeBy :: Pattern Double -> Pattern a -> Pattern a
 degradeBy = tParam _degradeBy
 
 _degradeBy :: Double -> Pattern a -> Pattern a
-_degradeBy x p = fmap fst $ filterValues ((> x) . snd) $ (,) <$> p <*> rand
+_degradeBy = _degradeByUsing rand
+
+-- Useful for manipulating random stream, e.g. to change 'seed'
+_degradeByUsing :: Pattern Double -> Double -> Pattern a -> Pattern a
+_degradeByUsing prand x p = fmap fst $ filterValues ((> x) . snd) $ (,) <$> p <* prand
 
 unDegradeBy :: Pattern Double -> Pattern a -> Pattern a
 unDegradeBy = tParam _unDegradeBy
 
 _unDegradeBy :: Double -> Pattern a -> Pattern a
-_unDegradeBy x p = fmap fst $ filterValues ((<= x) . snd) $ (,) <$> p <*> rand
+_unDegradeBy x p = fmap fst $ filterValues ((<= x) . snd) $ (,) <$> p <* rand
 
 degradeOverBy :: Int -> Pattern Double -> Pattern a -> Pattern a
-degradeOverBy i tx p = unwrap $ (\x -> fmap fst $ filterValues ((> x) . snd) $ (,) <$> p <*> fastRepeatCycles i rand) <$> slow (fromIntegral i) tx
+degradeOverBy i tx p = unwrap $ (\x -> fmap fst $ filterValues ((> x) . snd) $ (,) <$> p <* fastRepeatCycles i rand) <$> slow (fromIntegral i) tx
 
 
 {- | Use @sometimesBy@ to apply a given function "sometimes". For example, the
@@ -255,11 +261,14 @@ always = id
 
 {- | @someCyclesBy@ is a cycle-by-cycle version of @sometimesBy@. It has a
 `someCycles = someCyclesBy 0.5` alias -}
-someCyclesBy :: Double -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
-someCyclesBy x = when test
+someCyclesBy :: Pattern Double -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+someCyclesBy pd f p = do {d <- pd; _someCyclesBy d f p}
+
+_someCyclesBy :: Double -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+_someCyclesBy x = when test
   where test c = timeToRand (fromIntegral c :: Double) < x
 
-somecyclesBy :: Double -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+somecyclesBy :: Pattern Double -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
 somecyclesBy = someCyclesBy
 
 someCycles :: (Pattern a -> Pattern a) -> Pattern a -> Pattern a
@@ -868,19 +877,19 @@ _rot i pat = splitQueries $ pat {query = \st -> f st (query pat (st {arc = whole
         -- grouped together in the rotation?
         f st es = constrainEvents (arc st) $ shiftValues $ sort $ defragParts es
         shiftValues es | i >= 0 =
-                         zipWith (\(Event w p _) s -> Event w p s) es
+                         zipWith (\e s -> e {value = s}) es
                          (drop i $ cycle $ map value es)
                        | otherwise =
-                         zipWith (\(Event w p _) s -> Event w p s) es
+                         zipWith (\e s -> e{value = s}) es
                          (drop (length es - abs i) $ cycle $ map value es)
         wholeCycle (Arc s _) = Arc (sam s) (nextSam s)
         constrainEvents :: Arc -> [Event a] -> [Event a]
         constrainEvents a es = mapMaybe (constrainEvent a) es
         constrainEvent :: Arc -> Event a -> Maybe (Event a)
-        constrainEvent a (Event w p v) =
+        constrainEvent a e =
           do
-            p' <- subArc p a
-            return (Event w p' v)
+            p' <- subArc (part e) a
+            return e {part = p'}
 
 -- | @segment n p@: 'samples' the pattern @p@ at a rate of @n@
 -- events per cycle. Useful for turning a continuous pattern into a
@@ -900,6 +909,9 @@ discretise = segment
 randcat :: [Pattern a] -> Pattern a
 randcat ps = spread' rotL (_segment 1 $ (%1) . fromIntegral <$> (irand (length ps) :: Pattern Int)) (slowcat ps)
 
+wrandcat :: [(Pattern a, Double)] -> Pattern a
+wrandcat ps = unwrap $ wchooseBy (segment 1 rand) ps
+
 -- @fromNote p@: converts a pattern of human-readable pitch names
 -- into pitch numbers. For example, @"cs2"@ will be parsed as C Sharp
 -- in the 2nd octave with the result of @11@, and @"b-3"@ as
@@ -968,7 +980,7 @@ struct ps pv = filterJust $ (\a b -> if a then Just b else Nothing ) <$> ps <* p
 substruct :: Pattern String -> Pattern b -> Pattern b
 substruct s p = p {query = f}
   where f st =
-          concatMap ((\a' -> queryArc (compressArcTo a' p) a') . whole) (query s st)
+          concatMap ((\a' -> queryArc (compressArcTo a' p) a') . fromJust . whole) $ filter isDigital $ (query s st)
 
 randArcs :: Int -> Pattern [Arc]
 randArcs n =
@@ -986,8 +998,8 @@ randArcs n =
 
 -- TODO - what does this do? Something for @stripe@ ..
 randStruct :: Int -> Pattern Int
-randStruct n = splitQueries $ Pattern {nature = Digital, query = f}
-  where f st = map (\(a,b,c) -> Event a (fromJust b) c) $ filter (\(_,x,_) -> isJust x) as
+randStruct n = splitQueries $ Pattern {query = f}
+  where f st = map (\(a,b,c) -> Event (Context []) (Just a) (fromJust b) c) $ filter (\(_,x,_) -> isJust x) as
           where as = map (\(i, Arc s' e') ->
                     (Arc (s' + sam s) (e' + sam s),
                        subArc (Arc s e) (Arc (s' + sam s) (e' + sam s)), i)) $
@@ -997,7 +1009,10 @@ randStruct n = splitQueries $ Pattern {nature = Digital, query = f}
 
 -- TODO - what does this do?
 substruct' :: Pattern Int -> Pattern a -> Pattern a
-substruct' s p = p {query = \st -> concatMap (\(Event a' _ i) -> queryArc (compressArcTo a' (inside (pure $ 1/toRational(length (queryArc s (Arc (sam (start $ arc st)) (nextSam (start $ arc st)))))) (rotR (toRational i)) p)) a') (query s st)}
+substruct' s p = p {query = \st -> concatMap (f st) (query s st)}
+  where f st (Event c (Just a') _ i) = map (\e -> e {context = combineContexts [c, context e]}) $ queryArc (compressArcTo a' (inside (pure $ 1/toRational(length (queryArc s (Arc (sam (start $ arc st)) (nextSam (start $ arc st)))))) (rotR (toRational i)) p)) a'
+        -- Ignore analog events (ones without wholes)
+        f _ _ = []
 
 -- | @stripe n p@: repeats pattern @p@, @n@ times per cycle. So
 -- similar to @fast@, but with random durations. The repetitions will
@@ -1051,6 +1066,45 @@ rationals are required) -}
 lindenmayerI :: Num b => Int -> String -> String -> [b]
 lindenmayerI n r s = fmap (fromIntegral . digitToInt) $ lindenmayer n r s
 
+{- | @runMarkov n tmat xi seed@ generates a Markov chain (as a list) of length @n@
+using the transition matrix @tmat@ starting from initial state @xi@, starting
+with random numbers generated from @seed@
+Each entry in the chain is the index of state (starting from zero). 
+Each row of the matrix will be automatically normalized. For example:
+@
+runMarkov 8 [[2,3], [1,3]] 0 0
+@
+will produce a two-state chain 8 steps long, from initial state @0@, where the
+transition probability from state 0->0 is 2/5, 0->1 is 3/5, 1->0 is 1/4, and 
+1->1 is 3/4.  -}
+runMarkov :: Int -> [[Double]] -> Int -> Time -> [Int]
+runMarkov n tp xi seed = reverse $ (iterate (markovStep $ renorm) [xi])!! (n-1) where
+  markovStep tp' xs = (fromJust $ findIndex (r <=) $ scanl1 (+) (tp'!!(head xs))) : xs where
+    r = timeToRand $ seed + (fromIntegral . length) xs / fromIntegral n
+  renorm = [ map (/ sum x) x | x <- tp ]
+
+{- @markovPat n xi tp@ generates a one-cycle pattern of @n@ steps in a Markov
+chain starting from state @xi@ with transition matrix @tp@. Each row of the
+transition matrix is automatically normalized.  For example:
+@
+tidal> markovPat 8 1 [[3,5,2], [4,4,2], [0,1,0]]
+
+(0>⅛)|1
+(⅛>¼)|2
+(¼>⅜)|1
+(⅜>½)|1
+(½>⅝)|2
+(⅝>¾)|1
+(¾>⅞)|1
+(⅞>1)|0
+@ -}
+markovPat :: Pattern Int -> Pattern Int -> [[Double]] -> Pattern Int
+markovPat = tParam2 _markovPat
+
+_markovPat :: Int -> Int -> [[Double]] -> Pattern Int
+_markovPat n xi tp = splitQueries $ Pattern (\(State a@(Arc s _) _) -> 
+  queryArc (listToPat $ runMarkov n tp xi (sam s)) a)
+
 {-|
 Removes events from second pattern that don't start during an event from first.
 
@@ -1080,10 +1134,7 @@ d1 $ s (mask ("1 ~ 1 ~ 1 1 ~ 1")
 -}
 
 mask :: Pattern Bool -> Pattern a -> Pattern a
-mask maskpat pat = filterJust $ toMaybe <$> pat'
-  where pat' = matchManyToOne (flip const) maskpat pat
-        toMaybe (True, a) = Just a
-        toMaybe (False, _) = Nothing
+mask b p = const <$> p <* (filterValues id b)
 
 {-
 mask :: Pattern Bool -> Pattern b -> Pattern b
@@ -1101,7 +1152,7 @@ enclosingArc as = Arc (minimum (map start as)) (maximum (map stop as))
 stretch :: Pattern a -> Pattern a
 -- TODO - should that be whole or part?
 stretch p = splitQueries $ p {query = q}
-  where q st = query (zoomArc (cycleArc $ enclosingArc $ map whole $ query p (st {arc = Arc (sam s) (nextSam s)})) p) st
+  where q st = query (zoomArc (cycleArc $ enclosingArc $ map wholeOrPart $ query p (st {arc = Arc (sam s) (nextSam s)})) p) st
           where s = start $ arc st
 
 {- | `fit'` is a generalization of `fit`, where the list is instead constructed by using another integer pattern to slice up a given pattern.  The first argument is the number of cycles of that latter pattern to use when slicing.  It's easier to understand this with a few examples:
@@ -1165,8 +1216,8 @@ outside n = inside (1/n)
 loopFirst :: Pattern a -> Pattern a
 loopFirst p = splitQueries $ p {query = f}
   where f st = map
-          (\(Event w p' v) ->
-             Event (plus w) (plus p') v) $
+          (\(Event c w p' v) ->
+             Event c (plus <$> w) (plus p') v) $
           query p (st {arc = minus $ arc st})
           where minus = fmap (subtract (sam s))
                 plus = fmap (+ sam s)
@@ -1244,14 +1295,14 @@ _scramble n = _rearrangeWith (_segment (fromIntegral n) $ irand n) n
 randrun :: Int -> Pattern Int
 randrun 0 = silence
 randrun n' =
-  splitQueries $ Pattern Digital (\(State a@(Arc s _) _) -> events a $ sam s)
+  splitQueries $ Pattern (\(State a@(Arc s _) _) -> events a $ sam s)
   where events a seed = mapMaybe toEv $ zip arcs shuffled
           where shuffled = map snd $ sortOn fst $ zip rs [0 .. (n'-1)]
                 rs = timeToRands seed n'
                 arcs = zipWith Arc fractions (tail fractions)
                 fractions = map (+ (sam $ start a)) [0, 1 / fromIntegral n' .. 1]
                 toEv (a',v) = do a'' <- subArc a a'
-                                 return $ Event a' a'' v
+                                 return $ Event (Context []) (Just a') a'' v
 
 ur :: Time -> Pattern String -> [(String, Pattern a)] -> [(String, Pattern a -> Pattern a)] -> Pattern a
 ur t outer_p ps fs = _slow t $ unwrap $ adjust <$> timedValues (getPat . split <$> outer_p)
@@ -1266,7 +1317,7 @@ ur t outer_p ps fs = _slow t $ unwrap $ adjust <$> timedValues (getPat . split <
         transform _ _ = id
         transform' str (Arc s e) p = s `rotR` inside (pure $ 1/(e-s)) (matchF str) p
         matchF str = fromMaybe id $ lookup str fs
-        timedValues = withEvent (\(Event a a' v) -> Event a a' (a,v))
+        timedValues = withEvent (\(Event c (Just a) a' v) -> Event c (Just a) a' (a,v)) . filterDigital
 
 inhabit :: [(String, Pattern a)] -> Pattern String -> Pattern a
 inhabit ps p = squeezeJoin $ (\s -> fromMaybe silence $ lookup s ps) <$> p
@@ -1283,7 +1334,7 @@ spaceOut xs p = _slow (toRational $ sum xs) $ stack $ map (`compressArc` p) spac
 -- | @flatpat@ takes a Pattern of lists and pulls the list elements as
 -- separate Events
 flatpat :: Pattern [a] -> Pattern a
-flatpat p = p {query = concatMap (\(Event b b' xs) -> map (Event b b') xs) . query p}
+flatpat p = p {query = concatMap (\(Event c b b' xs) -> map (Event c b b') xs) . query p}
 
 -- | @layer@ takes a Pattern of lists and pulls the list elements as
 -- separate Events
@@ -1304,13 +1355,15 @@ arpWith :: ([EventF (ArcF Time) a] -> [EventF (ArcF Time) b]) -> Pattern a -> Pa
 arpWith f p = withEvents munge p
   where munge es = concatMap (spreadOut . f) (groupBy (\a b -> whole a == whole b) $ sortOn whole es)
         spreadOut xs = mapMaybe (\(n, x) -> shiftIt n (length xs) x) $ enumerate xs
-        shiftIt n d (Event (Arc s e) a' v) =
+        shiftIt n d (Event c (Just (Arc s e)) a' v) =
           do
             a'' <- subArc (Arc newS newE) a'
-            return (Event (Arc newS newE) a'' v)
+            return (Event c (Just $ Arc newS newE) a'' v)
           where newS = s + (dur * fromIntegral n)
                 newE = newS + dur
                 dur = (e - s) / fromIntegral d
+        -- TODO ignoring analog events.. Should we just leave them as-is?
+        shiftIt _ _ _ = Nothing
 
 arp :: Pattern String -> Pattern a -> Pattern a
 arp = tParam _arp
@@ -1372,12 +1425,34 @@ ply = tParam _ply
 _ply :: Int -> Pattern a -> Pattern a
 _ply n p = arpeggiate $ stack (replicate n p)
 
--- Uses the first (binary) pattern to switch between the following two
--- patterns.
+-- Like ply, but applies a function each time. The applications are compounded.
+plyWith :: (Ord t, Num t) => Pattern t -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+plyWith np f p = innerJoin $ (\n -> _plyWith n f p) <$> np
+
+_plyWith :: (Ord t, Num t) => t -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+_plyWith numPat f p = arpeggiate $ compound numPat
+  where compound n | n <= 1 = p
+                   | otherwise = overlay p (f $ compound $ n-1)
+
+-- | Uses the first (binary) pattern to switch between the following
+-- two patterns. The resulting structure comes from the source patterns, not the
+-- binary pattern. See also @stitch@.
 sew :: Pattern Bool -> Pattern a -> Pattern a -> Pattern a
-sew stitch p1 p2 = overlay (const <$> p1 <* a) (const <$> p2 <* b)
-  where a = filterValues id stitch
-        b = filterValues not stitch
+sew pb a b = overlay (mask pb a) (mask (inv pb) b)
+
+-- | Uses the first (binary) pattern to switch between the following
+-- two patterns. The resulting structure comes from the binary
+-- pattern, not the source patterns. See also @sew@.
+stitch :: Pattern Bool -> Pattern a -> Pattern a -> Pattern a
+stitch pb a b = overlay (struct pb a)  (struct (inv pb) b)
+
+-- | A binary pattern is used to conditionally apply a function to a
+-- source pattern. The function is applied when a @True@ value is
+-- active, and the pattern is let through unchanged when a @False@
+-- value is active. No events are let through where no binary values
+-- are active.
+while :: Pattern Bool -> (Pattern a -> Pattern a) -> Pattern a -> Pattern a
+while b f pat = sew b (f pat) pat
 
 stutter :: Integral i => i -> Time -> Pattern a -> Pattern a
 stutter n t p = stack $ map (\i -> (t * fromIntegral i) `rotR` p) [0 .. (n-1)]
@@ -1521,10 +1596,7 @@ d1 $ jux (iter 4) $ sound "arpy arpy:2*2"
 -}
 
 range :: Num a => Pattern a -> Pattern a -> Pattern a -> Pattern a
-range fromP toP p = do
-  from <- fromP
-  to <- toP
-  _range from to p
+range fromP toP p = (\from to v -> ((v * (to-from)) + from)) <$> fromP *> toP *> p
 
 _range :: (Functor f, Num b) => b -> b -> f b -> f b
 _range from to p = (+ from) . (* (to-from)) <$> p
@@ -1605,7 +1677,7 @@ _selectF f ps p =  (ps !! floor (max 0 (min 0.999999 f) * fromIntegral (length p
 
 -- | chooses between a list of functions, using a pattern of integers
 pickF :: Pattern Int -> [Pattern a -> Pattern a] -> Pattern a -> Pattern a
-pickF pi fs pat = innerJoin $ (\i -> _pickF i fs pat) <$> pi
+pickF pInt fs pat = innerJoin $ (\i -> _pickF i fs pat) <$> pInt
 
 _pickF :: Int -> [Pattern a -> Pattern a] -> Pattern a -> Pattern a
 _pickF i fs p =  (fs !!! i) p
@@ -1680,16 +1752,17 @@ inv = (not <$>)
 -- | Serialises a pattern so there's only one event playing at any one
 -- time, making it 'monophonic'. Events which start/end earlier are given priority.
 mono :: Pattern a -> Pattern a
-mono p = Pattern Digital $ \(State a cm) -> flatten $ query p (State a cm) where
+mono p = Pattern $ \(State a cm) -> flatten $ query p (State a cm) where
   flatten :: [Event a] -> [Event a]
   flatten = mapMaybe constrainPart . truncateOverlaps . sortOn whole
   truncateOverlaps [] = []
   truncateOverlaps (e:es) = e : truncateOverlaps (mapMaybe (snip e) es)
-  snip a b | start (whole b) >= stop (whole a) = Just b
-           | stop (whole b) <= stop (whole a) = Nothing
-           | otherwise = Just b {whole = Arc (stop $ whole a) (stop $ whole b)}
+  -- TODO - decide what to do about analog events..
+  snip a b | start (wholeOrPart b) >= stop (wholeOrPart a) = Just b
+           | stop (wholeOrPart b) <= stop (wholeOrPart a) = Nothing
+           | otherwise = Just b {whole = Just $ Arc (stop $ wholeOrPart a) (stop $ wholeOrPart b)}
   constrainPart :: Event a -> Maybe (Event a)
-  constrainPart e = do a <- subArc (whole e) (part e)
+  constrainPart e = do a <- subArc (wholeOrPart e) (part e)
                        return $ e {part = a}
 
 -- serialize the given pattern
@@ -1700,24 +1773,26 @@ mono p = Pattern Digital $ \(State a cm) -> flatten $ query p (State a cm) where
 -- if there is, check where we are in the 'whole' of the event, and use that to tween between the values of the event and the next event
 -- smooth :: Pattern Double -> Pattern Double
 
+-- TODO - test this with analog events
 smooth :: Fractional a => Pattern a -> Pattern a
-smooth p = Pattern Analog $ \st@(State a cm) -> tween st a $ query monoP (State (midArc a) cm)
+smooth p = Pattern $ \st@(State a cm) -> tween st a $ query monoP (State (midArc a) cm)
   where
     midArc a = Arc (mid (start a, stop a)) (mid (start a, stop a))
     tween _ _ [] = []
-    tween st queryA (e:_) = maybe [e {whole = queryA, part = queryA}] (tween' queryA) (nextV st)
+    tween st queryA (e:_) = maybe [e {whole = Just queryA, part = queryA}] (tween' queryA) (nextV st)
       where aStop = Arc (wholeStop e) (wholeStop e)
             nextEs st' = query monoP (st' {arc = aStop})
             nextV st' | null (nextEs st') = Nothing
                       | otherwise = Just $ value (head (nextEs st'))
             tween' queryA' v =
               [ Event
-                { whole = queryA'
+                { context = context e,
+                  whole = Just queryA'
                 , part = queryA'
                 , value = value e + ((v - value e) * pc)}
               ]
-            pc | delta' (whole e) == 0 = 0
-               | otherwise = fromRational $ (eventPartStart e - wholeStart e) / delta' (whole e)
+            pc | delta' (wholeOrPart e) == 0 = 0
+               | otherwise = fromRational $ (eventPartStart e - wholeStart e) / delta' (wholeOrPart e)
             delta' a = stop a - start a
     monoP = mono p
 
@@ -1780,17 +1855,33 @@ squeeze ipat pats = squeezeJoin $ (pats !!!) <$> ipat
 
 squeezeJoinUp :: Pattern (ControlPattern) -> ControlPattern
 squeezeJoinUp pp = pp {query = q}
-  where q st = concatMap
-          (\(Event w p v) ->
-             mapMaybe (munge w p) $ query (compressArc (cycleArc w) (v |* P.speed (pure $ fromRational $ 1/(stop w - start w)))) st {arc = p}
-          )
-          (query pp st)
-        munge oWhole oPart (Event iWhole iPart v) =
+  where q st = concatMap (f st) (query (filterDigital pp) st)
+        f st (Event c (Just w) p v) =
+          mapMaybe (munge c w p) $ query (compressArc (cycleArc w) (v |* P.speed (pure $ fromRational $ 1/(stop w - start w)))) st {arc = p}
+        -- already ignoring analog events, but for completeness..
+        f _ _ = []
+        munge co oWhole oPart (Event ci (Just iWhole) iPart v) =
           do w' <- subArc oWhole iWhole
              p' <- subArc oPart iPart
-             return (Event w' p' v)
+             return (Event (combineContexts [ci,co]) (Just w') p' v)
+        munge _ _ _ _ = Nothing
 
 chew :: Int -> Pattern Int -> ControlPattern  -> ControlPattern
 chew n ipat pat = (squeezeJoinUp $ zoompat <$> ipat) |/ P.speed (pure $ fromIntegral n)
   where zoompat i = zoom (i'/(fromIntegral n), (i'+1)/(fromIntegral n)) (pat)
            where i' = fromIntegral $ i `mod` n
+
+__binary :: Data.Bits.Bits b => Int -> b -> [Bool]
+__binary n num = map (testBit num) $ reverse [0 .. n-1]
+
+_binary :: Data.Bits.Bits b => Int -> b -> Pattern Bool
+_binary n num = listToPat $ __binary n num
+
+binaryN :: Int -> Pattern Int -> Pattern Bool
+binaryN n p = squeezeJoin $ _binary n <$> p
+
+binary :: Pattern Int -> Pattern Bool
+binary = binaryN 8
+
+ascii :: Pattern String -> Pattern Bool
+ascii p = squeezeJoin $ (listToPat . concatMap (__binary 8 . ord)) <$> p
